@@ -7,11 +7,9 @@ use gix::bstr::{BString, Utf8Error};
 use gix::object::tree::diff::change::Event;
 use gix::object::tree::diff::Action;
 use gix::prelude::ObjectIdExt;
-use gix::traverse::commit::Sorting;
+use gix::traverse::commit::simple::Sorting;
 use gix::{Commit, ObjectId};
-use regex::Regex;
 use std::collections::HashMap;
-use std::str::FromStr;
 use std::sync::atomic::{AtomicBool, AtomicUsize, Ordering};
 use std::sync::mpsc::{channel, Sender};
 use std::sync::Arc;
@@ -22,7 +20,7 @@ pub mod sig;
 
 pub fn traverse_commit_graph(
     repo: &gix::Repository,
-    no_bots: &Option<Option<MyRegex>>,
+    no_bots: Option<MyRegex>,
     max_churn_pool_size: Option<usize>,
     no_merges: bool,
 ) -> Result<GitMetrics> {
@@ -30,7 +28,6 @@ pub fn traverse_commit_graph(
     let mut time_of_first_commit = None;
     let mut number_of_commits_by_signature: HashMap<Sig, usize> = HashMap::new();
     let mailmap = repo.open_mailmap();
-    let bot_regex_pattern = get_no_bots_regex(no_bots)?;
     let has_commit_graph_traversal_ended = Arc::new(AtomicBool::default());
     let total_number_of_commits = Arc::new(AtomicUsize::default());
 
@@ -51,13 +48,15 @@ pub fn traverse_commit_graph(
 
     let (churn_thread, churn_tx) = get_churn_channel(
         repo,
+        &mailmap,
+        no_bots.clone(),
         &has_commit_graph_traversal_ended,
         &total_number_of_commits,
         max_churn_pool_size,
     )?;
 
     let author_threads = can_use_author_threads
-        .then(|| get_author_channel(repo, num_threads, &bot_regex_pattern, &mailmap));
+        .then(|| get_author_channel(repo, num_threads, no_bots.clone(), &mailmap));
 
     let mut count = 0;
     for commit in commit_iter {
@@ -73,7 +72,7 @@ pub fn traverse_commit_graph(
                 update_signature_counts(
                     &commit.object()?,
                     &mailmap,
-                    &bot_regex_pattern,
+                    no_bots.as_ref(),
                     &mut number_of_commits_by_signature,
                 )?;
             }
@@ -106,6 +105,8 @@ pub fn traverse_commit_graph(
     total_number_of_commits.store(count, Ordering::SeqCst);
     has_commit_graph_traversal_ended.store(true, Ordering::SeqCst);
 
+    drop(churn_tx);
+
     let (number_of_commits_by_file_path, churn_pool_size) =
         churn_thread.join().expect("never panics")?;
 
@@ -125,7 +126,7 @@ type NumberOfCommitsBySignature = HashMap<Sig, usize>;
 fn get_author_channel(
     repo: &gix::Repository,
     num_threads: usize,
-    bot_regex_pattern: &Option<MyRegex>,
+    bot_regex_pattern: Option<MyRegex>,
     mailmap: &gix::mailmap::Snapshot,
 ) -> (
     Vec<JoinHandle<Result<NumberOfCommitsBySignature>>>,
@@ -153,7 +154,7 @@ fn get_author_channel(
                         update_signature_counts(
                             &commit,
                             &mailmap,
-                            &bot_regex_pattern,
+                            bot_regex_pattern.as_ref(),
                             &mut number_of_commits_by_signature,
                         )?;
                     }
@@ -170,6 +171,8 @@ type ChurnPair = (NumberOfCommitsByFilepath, usize);
 
 fn get_churn_channel(
     repo: &gix::Repository,
+    mailmap: &gix::mailmap::Snapshot,
+    bot_regex_pattern: Option<MyRegex>,
     has_commit_graph_traversal_ended: &Arc<AtomicBool>,
     total_number_of_commits: &Arc<AtomicUsize>,
     max_churn_pool_size: Option<usize>,
@@ -177,6 +180,8 @@ fn get_churn_channel(
     let (tx, rx) = channel::<gix::hash::ObjectId>();
     let thread = std::thread::spawn({
         let repo = repo.clone();
+        let mailmap = mailmap.clone();
+        let bot_regex_pattern = bot_regex_pattern.clone();
         let has_commit_graph_traversal_ended = has_commit_graph_traversal_ended.clone();
         let total_number_of_commits = total_number_of_commits.clone();
         move || -> Result<_> {
@@ -184,6 +189,9 @@ fn get_churn_channel(
             let mut number_of_diffs_computed = 0;
             while let Ok(commit_id) = rx.recv() {
                 let commit = repo.find_object(commit_id)?.into_commit();
+                if is_bot_commit(&commit, &mailmap, bot_regex_pattern.as_ref())? {
+                    continue;
+                }
                 compute_diff_with_parent(&mut number_of_commits_by_file_path, &commit, &repo)?;
                 number_of_diffs_computed += 1;
                 if should_break(
@@ -221,7 +229,7 @@ fn should_break(
 fn update_signature_counts(
     commit: &gix::Commit,
     mailmap: &gix::mailmap::Snapshot,
-    bot_regex_pattern: &Option<MyRegex>,
+    bot_regex_pattern: Option<&MyRegex>,
     number_of_commits_by_signature: &mut HashMap<Sig, usize>,
 ) -> Result<()> {
     let sig = mailmap.resolve(commit.author()?);
@@ -273,21 +281,21 @@ fn compute_diff_with_parent(
     Ok(())
 }
 
-fn get_no_bots_regex(no_bots: &Option<Option<MyRegex>>) -> Result<Option<MyRegex>> {
-    let reg = if let Some(r) = no_bots.clone() {
-        match r {
-            Some(p) => Some(p),
-            None => Some(MyRegex(Regex::from_str(r"(?:-|\s)[Bb]ot$|\[[Bb]ot\]")?)),
-        }
+fn is_bot_commit(
+    commit: &Commit,
+    mailmap: &gix::mailmap::Snapshot,
+    bot_regex_pattern: Option<&MyRegex>,
+) -> Result<bool> {
+    if bot_regex_pattern.is_some() {
+        let sig = mailmap.resolve(commit.author()?);
+        Ok(is_bot(&sig.name, bot_regex_pattern))
     } else {
-        None
-    };
-
-    Ok(reg)
+        Ok(false)
+    }
 }
 
-fn is_bot(author_name: &BString, bot_regex_pattern: &Option<MyRegex>) -> bool {
-    bot_regex_pattern.as_ref().map_or(false, |regex| {
+fn is_bot(author_name: &BString, bot_regex_pattern: Option<&MyRegex>) -> bool {
+    bot_regex_pattern.map_or(false, |regex| {
         regex.0.is_match(author_name.to_str_lossy().as_ref())
     })
 }
@@ -295,28 +303,9 @@ fn is_bot(author_name: &BString, bot_regex_pattern: &Option<MyRegex>) -> bool {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::cli::NO_BOTS_DEFAULT_REGEX_PATTERN;
     use rstest::rstest;
-
-    #[test]
-    fn test_get_no_bots_regex() -> Result<()> {
-        // Test case 1: no_bots is None
-        let no_bots: Option<Option<MyRegex>> = None;
-        let result = get_no_bots_regex(&no_bots)?;
-        assert_eq!(result, None);
-
-        // Test case 2: no_bots is Some(None)
-        let no_bots: Option<Option<MyRegex>> = Some(None);
-        let result = get_no_bots_regex(&no_bots)?;
-        assert_eq!(result.unwrap().0.as_str(), r"(?:-|\s)[Bb]ot$|\[[Bb]ot\]");
-
-        // Test case 3: no_bots is Some(Some(regex))
-        let regex = MyRegex(Regex::new(r"foo")?);
-        let no_bots: Option<Option<MyRegex>> = Some(Some(regex));
-        let result = get_no_bots_regex(&no_bots)?;
-        assert_eq!(result.unwrap().0.as_str(), "foo");
-
-        Ok(())
-    }
+    use std::str::FromStr;
 
     #[rstest]
     #[case("John Doe", false)]
@@ -325,9 +314,9 @@ mod tests {
     #[case("foo-bot", true)]
     #[case("bot", false)]
     fn test_is_bot(#[case] author_name: &str, #[case] expected: bool) -> Result<()> {
-        let no_bots: Option<Option<MyRegex>> = Some(None);
-        let regex = get_no_bots_regex(&no_bots)?;
-        assert_eq!(is_bot(&author_name.into(), &regex), expected);
+        let from_str = MyRegex::from_str(NO_BOTS_DEFAULT_REGEX_PATTERN);
+        let no_bots: Option<MyRegex> = Some(from_str?);
+        assert_eq!(is_bot(&author_name.into(), no_bots.as_ref()), expected);
         Ok(())
     }
 
